@@ -21,7 +21,9 @@
 **  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 **  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
 */
-
+                             /* ``Study it forever and you'll still wonder.
+                                  Fly it once and you'll know.''
+                                                       -- Henry Spencer */
 #include "pth_p.h"
 
 /* set cancellation state */
