@@ -21,7 +21,10 @@
 **  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 **  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
 */
-
+                             /* ``Those of you who think they
+                                  know everything are very annoying
+                                  to those of us who do.''
+                                                  -- Unknown       */
 #include "pth_p.h"
 
 #if cpp
@@ -75,13 +78,13 @@ pth_event_t pth_event(unsigned long spec, ...)
             pth_key_create(ev_key, pth_event_destructor);
         ev = (pth_event_t)pth_key_getdata(*ev_key);
         if (ev == NULL) {
-            ev = malloc(sizeof(struct pth_event_st));
+            ev = (pth_event_t)malloc(sizeof(struct pth_event_st));
             pth_key_setdata(*ev_key, ev);
         }
     }
     else {
         /* allocate new dynamic event structure */
-        ev = malloc(sizeof(struct pth_event_st));
+        ev = (pth_event_t)malloc(sizeof(struct pth_event_st));
     }
     if (ev == NULL)
         return NULL;
@@ -169,16 +172,16 @@ pth_event_t pth_event(unsigned long spec, ...)
         pth_t tid = va_arg(ap, pth_t);
         int goal;
         ev->ev_type = PTH_EVENT_TID;
-        switch (spec & (PTH_UNTIL_TID_NEW|\
-                        PTH_UNTIL_TID_READY|\
-                        PTH_UNTIL_TID_WAITING|\
-                        PTH_UNTIL_TID_DEAD)) {
-            case PTH_UNTIL_TID_NEW:     goal = PTH_STATE_NEW;     break;
-            case PTH_UNTIL_TID_READY:   goal = PTH_STATE_READY;   break;
-            case PTH_UNTIL_TID_WAITING: goal = PTH_STATE_WAITING; break;
-            case PTH_UNTIL_TID_DEAD:    goal = PTH_STATE_DEAD;    break;
-            default:                    goal = PTH_STATE_READY;
-        }
+        if (spec & PTH_UNTIL_TID_NEW)
+            goal = PTH_STATE_NEW;
+        else if (spec & PTH_UNTIL_TID_READY)
+            goal = PTH_STATE_READY;
+        else if (spec & PTH_UNTIL_TID_WAITING)
+            goal = PTH_STATE_WAITING;
+        else if (spec & PTH_UNTIL_TID_DEAD)
+            goal = PTH_STATE_DEAD;
+        else
+            goal = PTH_STATE_READY;
         ev->ev_goal = goal;
         ev->ev_args.TID.tid = tid;
     }
@@ -394,8 +397,8 @@ int pth_wait(pth_event_t ev_ring)
     ev = ev_ring;
     do {
         ev->ev_occurred = FALSE;
-        ev = ev->ev_next;
         pth_debug2("pth_wait: waiting event 0x%lx", (unsigned long)ev);
+        ev = ev->ev_next;
     } while (ev != ev_ring);
 
     /* link event ring to current thread */
