@@ -21,6 +21,10 @@
 **  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 **  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
 */
+                             /* ``The difference between a computer
+                                  industry job and open-source software
+                                  hacking is about 30 hours a week.''
+                                         -- Ralf S. Engelschall     */
 
 /*
  *  These functions used by the applications instead of the
@@ -143,6 +147,9 @@ int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds,
     fd_set *rtmp, *wtmp, *etmp;
     int selected;
     int rc;
+    
+    pth_implicit_init();
+    pth_debug2("pth_select_ev: called from thread \"%s\"", pth_current->name);
 
     /* first deal with the special situation of a plain microsecond delay */
     if (nfd == 0 && rfds == NULL && wfds == NULL && efds == NULL && timeout != NULL) {
@@ -166,10 +173,14 @@ int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds,
                     return_errno(-1, EINTR);
             }
         }
+        /* POSIX compliance */
+        if (rfds != NULL) FD_ZERO(rfds);
+        if (wfds != NULL) FD_ZERO(wfds);
+        if (efds != NULL) FD_ZERO(efds);
         return 0;
     }
 
-    /* first directly poll filedescriptor sets to avoid unneccessary 
+    /* now directly poll filedescriptor sets to avoid unneccessary 
        (and resource consuming because of context switches, etc) event
        handling through the scheduler. We've to be carefully here, because not
        all platforms guarranty us that the sets are unmodified when an error
@@ -201,11 +212,18 @@ int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds,
             *efds = espare;
         return rc;
     }
-    if (rc == 0 && timeout != NULL)
-        if (pth_time_cmp(timeout, PTH_TIME_ZERO) == 0)
+    if (rc == 0 && timeout != NULL) {
+        if (pth_time_cmp(timeout, PTH_TIME_ZERO) == 0) {
+            /* POSIX compliance */
+            if (rfds != NULL) FD_ZERO(rfds);
+            if (wfds != NULL) FD_ZERO(wfds);
+            if (efds != NULL) FD_ZERO(efds);
             return 0;
+        }
+    }
 
     /* suspend current thread until one fd is ready or the timeout occurred */
+    rc = -1;
     ev = ev_select = pth_event(PTH_EVENT_SELECT|PTH_MODE_STATIC, 
                                &ev_key_select, &rc, nfd, rfds, wfds, efds);
     ev_timeout = NULL;
@@ -223,8 +241,14 @@ int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds,
         selected = TRUE;
     if (timeout != NULL) {
         pth_event_isolate(ev_timeout);
-        if (pth_event_occurred(ev_timeout))
+        if (pth_event_occurred(ev_timeout)) {
             selected = TRUE;
+            /* POSIX compliance */
+            if (rfds != NULL) FD_ZERO(rfds);
+            if (wfds != NULL) FD_ZERO(wfds);
+            if (efds != NULL) FD_ZERO(efds);
+            rc = 0;
+        }
     }
     if (ev_extra != NULL && !selected)
         return_errno(-1, EINTR);
@@ -244,9 +268,13 @@ int pth_poll_ev(struct pollfd *pfd, unsigned int nfd, int timeout, pth_event_t e
 {
     fd_set rfds, wfds, efds;
     struct timeval tv, *ptv;
-    int maxfd, rc, i, ok;
+    int maxfd, rc, ok;
+    unsigned int i;
     char data[64];
 
+    pth_implicit_init();
+    pth_debug2("pth_poll_ev: called from thread \"%s\"", pth_current->name);
+
     /* poll(2) semantics */
     if (pfd == NULL)
         return_errno(-1, EFAULT);
@@ -378,11 +406,15 @@ int pth_accept_ev(int s, struct sockaddr *addr, int *addrlen, pth_event_t ev_ext
 {
     pth_event_t ev;
     static pth_key_t ev_key = PTH_KEY_INIT;
+    int fdmode;
     int rv;
 
     pth_implicit_init();
     pth_debug2("pth_accept_ev: enter from thread \"%s\"", pth_current->name);
 
+    /* force filedescriptor into non-blocking mode */
+    fdmode = pth_fdmode(s, PTH_FDMODE_NONBLOCK);
+
     /* poll socket via accept */
     ev = NULL;
     while ((rv = pth_sc(accept)(s, addr, addrlen)) == -1
@@ -398,10 +430,20 @@ int pth_accept_ev(int s, struct sockaddr *addr, int *addrlen, pth_event_t ev_ext
         /* check for the extra events */
         if (ev_extra != NULL) {
             pth_event_isolate(ev);
-            if (!pth_event_occurred(ev))
+            if (!pth_event_occurred(ev)) {
+                pth_fdmode(s, fdmode);
                 return_errno(-1, EINTR);
+            }
         }
     }
+
+    /* restore filedescriptor mode */
+    errno_safe(
+        pth_fdmode(s, fdmode); 
+        if (rv != -1)
+            pth_fdmode(rv, fdmode);
+    );
+
     pth_debug2("pth_accept_ev: leave to thread \"%s\"", pth_current->name);
     return rv;
 }
@@ -428,33 +470,41 @@ ssize_t pth_read_ev(int fd, void *buf, size_t nbytes, pth_event_t ev_extra)
     if (nbytes == 0)
         return 0;
 
-    /* first directly poll filedescriptor for readability 
-       to avoid unneccessary (and resource consuming because of context
-       switches, etc) event handling through the scheduler */
-    FD_ZERO(&fds);
-    FD_SET(fd, &fds);
-    delay.tv_sec  = 0; 
-    delay.tv_usec = 0;
-    while ((n = pth_sc(select)(fd+1, &fds, NULL, NULL, &delay)) < 0 
-           && errno == EINTR) ; 
+    /* poll filedescriptor when not already in non-blocking operation */
+    if (pth_fdmode(fd, PTH_FDMODE_POLL) == PTH_FDMODE_BLOCK) {
 
-    /* when filedescriptor is still not readable, 
-       let thread sleep until it is or event occurs */
-    if (n < 1) {
-        ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_READABLE|PTH_MODE_STATIC, &ev_key, fd);
-        if (ev_extra != NULL)
-            pth_event_concat(ev, ev_extra, NULL);
-        n = pth_wait(ev);
-        if (ev_extra != NULL) {
-            pth_event_isolate(ev);
-            if (!pth_event_occurred(ev))
-                return_errno(-1, EINTR);
+        /* now directly poll filedescriptor for readability 
+           to avoid unneccessary (and resource consuming because of context
+           switches, etc) event handling through the scheduler */
+        FD_ZERO(&fds);
+        FD_SET(fd, &fds);
+        delay.tv_sec  = 0; 
+        delay.tv_usec = 0;
+        while ((n = pth_sc(select)(fd+1, &fds, NULL, NULL, &delay)) < 0 
+               && errno == EINTR) ; 
+
+        /* when filedescriptor is still not readable, 
+           let thread sleep until it is or the extra event occurs */
+        if (n < 1) {
+            ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_READABLE|PTH_MODE_STATIC, &ev_key, fd);
+            if (ev_extra != NULL)
+                pth_event_concat(ev, ev_extra, NULL);
+            n = pth_wait(ev);
+            if (ev_extra != NULL) {
+                pth_event_isolate(ev);
+                if (!pth_event_occurred(ev))
+                    return_errno(-1, EINTR);
+            }
         }
     }
 
-    /* now perform the actual read */
+    /* Now perform the actual read. We're now guarrantied to not block,
+       either because we were already in non-blocking mode or we determined
+       above by polling that the next read(2) call will not block.  But keep
+       in mind, that only 1 next read(2) call is guarrantied to not block
+       (except for the EINTR situation). */
     while ((n = pth_sc(read)(fd, buf, nbytes)) < 0 
-           && errno == EINTR) ;
+           && errno == EINTR) ;                    
 
     pth_debug2("pth_read_ev: leave to thread \"%s\"", pth_current->name);
     return n;
@@ -473,6 +523,9 @@ ssize_t pth_write_ev(int fd, const void *buf, size_t nbytes, pth_event_t ev_extr
     pth_event_t ev;
     static pth_key_t ev_key = PTH_KEY_INIT;
     fd_set fds;
+    int fdmode;
+    ssize_t rv;
+    ssize_t s;
     int n;
 
     pth_implicit_init();
@@ -482,36 +535,71 @@ ssize_t pth_write_ev(int fd, const void *buf, size_t nbytes, pth_event_t ev_extr
     if (nbytes == 0)
         return 0;
 
-    /* first directly poll filedescriptor for writeability 
-       to avoid unneccessary (and resource consuming because of context
-       switches, etc) event handling through the scheduler */
-    FD_ZERO(&fds);
-    FD_SET(fd, &fds);
-    delay.tv_sec  = 0; 
-    delay.tv_usec = 0;
-    while ((n = pth_sc(select)(fd+1, NULL, &fds, NULL, &delay)) < 0 
-           && errno == EINTR) ; 
+    /* force filedescriptor into non-blocking mode */
+    fdmode = pth_fdmode(fd, PTH_FDMODE_NONBLOCK);
 
-    /* when filedescriptor is still not writeable, 
-       let thread sleep until it is or event occurs */
-    if (n < 1) {
-        ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_WRITEABLE|PTH_MODE_STATIC, &ev_key, fd);
-        if (ev_extra != NULL)
-            pth_event_concat(ev, ev_extra, NULL);
-        pth_wait(ev);
-        if (ev_extra != NULL) {
-            pth_event_isolate(ev);
-            if (!pth_event_occurred(ev))
-                return_errno(-1, EINTR);
+    /* poll filedescriptor when not already in non-blocking operation */
+    if (fdmode != PTH_FDMODE_NONBLOCK) {
+
+        /* now directly poll filedescriptor for writeability 
+           to avoid unneccessary (and resource consuming because of context
+           switches, etc) event handling through the scheduler */
+        FD_ZERO(&fds);
+        FD_SET(fd, &fds);
+        delay.tv_sec  = 0; 
+        delay.tv_usec = 0;
+        while ((n = pth_sc(select)(fd+1, NULL, &fds, NULL, &delay)) < 0 
+               && errno == EINTR) ; 
+
+        rv = 0;
+        for (;;) {
+            /* when filedescriptor is still not writeable, 
+               let thread sleep until it is or event occurs */
+            if (n < 1) {
+                ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_WRITEABLE|PTH_MODE_STATIC, &ev_key, fd);
+                if (ev_extra != NULL)
+                    pth_event_concat(ev, ev_extra, NULL);
+                pth_wait(ev);
+                if (ev_extra != NULL) {
+                    pth_event_isolate(ev);
+                    if (!pth_event_occurred(ev)) {
+                        pth_fdmode(fd, fdmode);
+                        return_errno(-1, EINTR);
+                    }
+                }
+            }
+
+            /* now perform the actual write operation */
+            while ((s = pth_sc(write)(fd, buf, nbytes)) < 0 
+                   && errno == EINTR) ;
+            if (s > 0)
+                rv += s;
+
+            /* although we're physically now in non-blocking mode,
+               iterate unless all data is written or an error occurs, because
+               we've to mimic the usual blocking I/O behaviour of write(2). */
+            if (s > 0 && s < (ssize_t)nbytes) {
+                nbytes -= s;
+                buf = (void *)((char *)buf + s);
+                n = 0;
+                continue;
+            }
+
+            /* stop looping */
+            break;
         }
     }
+    else {
+        /* Now perform the actual write operation */
+        while ((rv = pth_sc(write)(fd, buf, nbytes)) < 0 
+               && errno == EINTR) ;
+    }
 
-    /* now perform the actual write */
-    while ((n = pth_sc(write)(fd, buf, nbytes)) < 0 
-           && errno == EINTR) ;
+    /* restore filedescriptor mode */
+    errno_safe( pth_fdmode(fd, fdmode) );
 
     pth_debug2("pth_write_ev: leave to thread \"%s\"", pth_current->name);
-    return n;
+    return rv;
 }
 
 /* Pth variant of waitpid(2) */
@@ -560,34 +648,42 @@ ssize_t pth_readv_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t ev
     pth_debug2("pth_readv_ev: enter from thread \"%s\"", pth_current->name);
 
     /* POSIX compliance */
-    if (iovcnt <= 0)
+    if (iovcnt <= 0 || iovcnt > UIO_MAXIOV)
         return_errno(-1, EINVAL);
 
-    /* first directly poll filedescriptor for readability 
-       to avoid unneccessary (and resource consuming because of context
-       switches, etc) event handling through the scheduler */
-    FD_ZERO(&fds);
-    FD_SET(fd, &fds);
-    delay.tv_sec  = 0; 
-    delay.tv_usec = 0;
-    while ((n = pth_sc(select)(fd+1, &fds, NULL, NULL, &delay)) < 0 
-           && errno == EINTR) ; 
+    /* poll filedescriptor when not already in non-blocking operation */
+    if (pth_fdmode(fd, PTH_FDMODE_POLL) == PTH_FDMODE_BLOCK) {
 
-    /* when filedescriptor is still not readable, 
-       let thread sleep until it is or event occurs */
-    if (n < 1) {
-        ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_READABLE|PTH_MODE_STATIC, &ev_key, fd);
-        if (ev_extra != NULL)
-            pth_event_concat(ev, ev_extra, NULL);
-        n = pth_wait(ev);
-        if (ev_extra != NULL) {
-            pth_event_isolate(ev);
-            if (!pth_event_occurred(ev))
-                return_errno(-1, EINTR);
+        /* first directly poll filedescriptor for readability 
+           to avoid unneccessary (and resource consuming because of context
+           switches, etc) event handling through the scheduler */
+        FD_ZERO(&fds);
+        FD_SET(fd, &fds);
+        delay.tv_sec  = 0; 
+        delay.tv_usec = 0;
+        while ((n = pth_sc(select)(fd+1, &fds, NULL, NULL, &delay)) < 0 
+               && errno == EINTR) ; 
+
+        /* when filedescriptor is still not readable, 
+           let thread sleep until it is or event occurs */
+        if (n < 1) {
+            ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_READABLE|PTH_MODE_STATIC, &ev_key, fd);
+            if (ev_extra != NULL)
+                pth_event_concat(ev, ev_extra, NULL);
+            n = pth_wait(ev);
+            if (ev_extra != NULL) {
+                pth_event_isolate(ev);
+                if (!pth_event_occurred(ev))
+                    return_errno(-1, EINTR);
+            }
         }
     }
 
-    /* now perform the actual read */
+    /* Now perform the actual read. We're now guarrantied to not block,
+       either because we were already in non-blocking mode or we determined
+       above by polling that the next read(2) call will not block.  But keep
+       in mind, that only 1 next read(2) call is guarrantied to not block
+       (except for the EINTR situation). */
 #if PTH_FAKE_RWV
     while ((n = pth_readv_faked(fd, iov, iovcnt)) < 0 
            && errno == EINTR) ;
@@ -601,7 +697,7 @@ ssize_t pth_readv_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t ev
 }
 
 /* A faked version of readv(2) */
-intern size_t pth_readv_faked(int fd, const struct iovec *iov, int iovcnt)
+intern ssize_t pth_readv_faked(int fd, const struct iovec *iov, int iovcnt)
 {
     char *buffer;
     size_t bytes, copy, rv;
@@ -611,17 +707,17 @@ intern size_t pth_readv_faked(int fd, const struct iovec *iov, int iovcnt)
     bytes = 0;
     for (i = 0; i < iovcnt; i++) {
         if (iov[i].iov_len <= 0)
-            return_errno(-1, EINVAL);
+            return_errno((ssize_t)(-1), EINVAL);
         bytes += iov[i].iov_len;
     }
     if (bytes <= 0)
-        return_errno(-1, EINVAL);
+        return_errno((ssize_t)(-1), EINVAL);
 
     /* allocate a temporary buffer */
-    if ((buffer = malloc(bytes)) == NULL)
-        return -1;
+    if ((buffer = (char *)malloc(bytes)) == NULL)
+        return (ssize_t)(-1);
     
-    /* read data into temporary buffer */
+    /* read data into temporary buffer (caller guarrantied us to not block) */
     rv = pth_sc(read)(fd, buffer, bytes);
 
     /* scatter read data into callers vector */
@@ -657,54 +753,161 @@ ssize_t pth_writev_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t e
     pth_event_t ev;
     static pth_key_t ev_key = PTH_KEY_INIT;
     fd_set fds;
+    int fdmode;
+    struct iovec *liov;
+    int liovcnt;
+    size_t nbytes;
+    ssize_t rv;
+    ssize_t s;
     int n;
 
     pth_implicit_init();
     pth_debug2("pth_writev_ev: enter from thread \"%s\"", pth_current->name);
 
     /* POSIX compliance */
-    if (iovcnt <= 0)
+    if (iovcnt <= 0 || iovcnt > UIO_MAXIOV)
         return_errno(-1, EINVAL);
+    
+    /* force filedescriptor into non-blocking mode */
+    fdmode = pth_fdmode(fd, PTH_FDMODE_NONBLOCK);
 
-    /* first directly poll filedescriptor for writeability 
-       to avoid unneccessary (and resource consuming because of context
-       switches, etc) event handling through the scheduler */
-    FD_ZERO(&fds);
-    FD_SET(fd, &fds);
-    delay.tv_sec  = 0; 
-    delay.tv_usec = 0;
-    while ((n = pth_sc(select)(fd+1, NULL, &fds, NULL, &delay)) < 0 
-           && errno == EINTR) ; 
+    /* poll filedescriptor when not already in non-blocking operation */
+    if (fdmode != PTH_FDMODE_NONBLOCK) {
 
-    /* when filedescriptor is still not writeable, 
-       let thread sleep until it is or event occurs */
-    if (n < 1) {
-        ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_WRITEABLE|PTH_MODE_STATIC, &ev_key, fd);
-        if (ev_extra != NULL)
-            pth_event_concat(ev, ev_extra, NULL);
-        pth_wait(ev);
-        if (ev_extra != NULL) {
-            pth_event_isolate(ev);
-            if (!pth_event_occurred(ev))
-                return_errno(-1, EINTR);
+        /* init return value and number of bytes to write */
+        rv      = 0;
+        nbytes  = pth_writev_iov_bytes(iov, iovcnt);
+
+        /* init local iovec structure */
+        liov    = NULL;
+        liovcnt = 0;
+        pth_writev_iov_advance(iov, iovcnt, 0, &liov, &liovcnt);
+
+        /* first directly poll filedescriptor for writeability 
+           to avoid unneccessary (and resource consuming because of context
+           switches, etc) event handling through the scheduler */
+        FD_ZERO(&fds);
+        FD_SET(fd, &fds);
+        delay.tv_sec  = 0; 
+        delay.tv_usec = 0;
+        while ((n = pth_sc(select)(fd+1, NULL, &fds, NULL, &delay)) < 0 
+               && errno == EINTR) ; 
+
+        for (;;) {
+            /* when filedescriptor is still not writeable, 
+               let thread sleep until it is or event occurs */
+            if (n < 1) {
+                ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_WRITEABLE|PTH_MODE_STATIC, &ev_key, fd);
+                if (ev_extra != NULL)
+                    pth_event_concat(ev, ev_extra, NULL);
+                pth_wait(ev);
+                if (ev_extra != NULL) {
+                    pth_event_isolate(ev);
+                    if (!pth_event_occurred(ev)) {
+                        pth_fdmode(fd, fdmode);
+                        return_errno(-1, EINTR);
+                    }
+                }
+            }
+
+            /* Now perform the actual write operation */
+#if PTH_FAKE_RWV
+            while ((s = pth_writev_faked(fd, liov, liovcnt)) < 0 
+                   && errno == EINTR) ;
+#else
+            while ((s = pth_sc(writev)(fd, liov, liovcnt)) < 0 
+                   && errno == EINTR) ;
+#endif
+            if (s > 0)
+                rv += s;
+
+            /* although we're physically now in non-blocking mode,
+               iterate unless all data is written or an error occurs, because
+               we've to mimic the usual blocking I/O behaviour of writev(2) */
+            if (s > 0 && s < (ssize_t)nbytes) {
+                nbytes -= s;
+                pth_writev_iov_advance(iov, iovcnt, n, &liov, &liovcnt);
+                n = 0;
+                continue;
+            }
+
+            /* stop looping */
+            break;
         }
     }
-
-    /* now perform the actual write */
+    else {
+        /* Now perform the actual write operation */
 #if PTH_FAKE_RWV
-    while ((n = pth_writev_faked(fd, iov, iovcnt)) < 0 
-           && errno == EINTR) ;
+        while ((rv = pth_writev_faked(fd, iov, iovcnt)) < 0 
+               && errno == EINTR) ;
 #else
-    while ((n = pth_sc(writev)(fd, iov, iovcnt)) < 0 
-           && errno == EINTR) ;
+        while ((rv = pth_sc(writev)(fd, iov, iovcnt)) < 0 
+               && errno == EINTR) ;
 #endif
+    }
+
+    /* restore filedescriptor mode */
+    errno_safe( pth_fdmode(fd, fdmode) );
 
     pth_debug2("pth_writev_ev: leave to thread \"%s\"", pth_current->name);
-    return n;
+    return rv;
+}
+
+/* calculate number of bytes in a struct iovec */
+intern ssize_t pth_writev_iov_bytes(const struct iovec *iov, int iovcnt)
+{
+    ssize_t bytes;
+    int i;
+
+    bytes = 0;
+    for (i = 0; i < iovcnt; i++) {
+        if (iov[i].iov_len <= 0)
+            continue;
+        bytes += iov[i].iov_len;
+    }
+    return bytes;
+}
+
+/* advance the virtual pointer of a struct iov */
+intern void pth_writev_iov_advance(const struct iovec *riov, int riovcnt, size_t advance,
+                                   struct iovec **liov, int *liovcnt)
+{
+    static struct iovec siov[UIO_MAXIOV];
+    int i;
+
+    if (*liov == NULL && *liovcnt == 0) {
+        /* initialize with real (const) structure on first step */
+        *liov = (struct iovec *)riov;
+        *liovcnt = riovcnt;
+    }
+    if (advance > 0) {
+        if (*liov == riov && *liovcnt == riovcnt) {
+            /* reinitialize with a copy to be able to adjust it */
+            *liov = &siov[0];
+            for (i = 0; i < riovcnt; i++) {
+                siov[i].iov_base = riov[i].iov_base;
+                siov[i].iov_len  = riov[i].iov_len;
+            }
+        }
+        /* advance the virtual pointer */
+        while (*liovcnt > 0 && advance > 0) {
+            if ((*liov)->iov_len > advance) {
+                (*liov)->iov_base = (void *)((char *)((*liov)->iov_base) + advance);
+                (*liov)->iov_len  -= advance;
+                break;
+            }
+            else {
+                advance -= (*liov)->iov_len;
+                (*liovcnt)--;
+                (*liov)++;
+            }
+        }
+    }
+    return;
 }
 
 /* A faked version of writev(2) */
-intern size_t pth_writev_faked(int fd, const struct iovec *iov, int iovcnt)
+intern ssize_t pth_writev_faked(int fd, const struct iovec *iov, int iovcnt)
 {
     char *buffer, *cp;
     size_t bytes, to_copy, copy, rv;
@@ -714,15 +917,15 @@ intern size_t pth_writev_faked(int fd, const struct iovec *iov, int iovcnt)
     bytes = 0;
     for (i = 0; i < iovcnt; i++) {
         if (iov[i].iov_len <= 0)
-            return_errno(-1, EINVAL);
+            return_errno((ssize_t)(-1), EINVAL);
         bytes += iov[i].iov_len;
     }
     if (bytes <= 0)
-        return_errno(-1, EINVAL);
+        return_errno((ssize_t)(-1), EINVAL);
 
     /* allocate a temporary buffer to hold the data */
-    if ((buffer = malloc(bytes)) == NULL)
-        return -1;
+    if ((buffer = (char *)malloc(bytes)) == NULL)
+        return (ssize_t)(-1);
 
     /* concatenate the data from callers vector into buffer */
     to_copy = bytes;
@@ -735,7 +938,7 @@ intern size_t pth_writev_faked(int fd, const struct iovec *iov, int iovcnt)
              break;
     }
 
-    /* write continuous chunck of data */
+    /* write continuous chunck of data (caller guarrantied us to not block) */
     rv = pth_sc(write)(fd, buffer, bytes);
 
     /* remove the temporary buffer */ 
