@@ -21,10 +21,14 @@
 **  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 **  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
 */
-
+                             /* ``It took me fifteen years to discover
+                                  I had no talent for programming, but
+                                  I couldn't give it up because by that
+                                  time I was too famous.''
+                                            -- Unknown                */
 #include "pth_p.h"
 
-/* return the hexadecial Pth library version number */
+/* return the hexadecimal Pth library version number */
 long pth_version(void)
 {
     return PTH_VERSION;
@@ -232,7 +236,7 @@ pth_t pth_spawn(pth_attr_t attr, void *(*func)(void*), void *arg)
     /* initialize the machine context of this new thread */
     if (t->stacksize > 0) /* the "main thread" (indicated by == 0) is special! */
         if (!pth_mctx_set(&t->mctx, pth_spawn_trampoline,
-                          t->stack, (void *)((char *)t->stack+t->stacksize)))
+                          t->stack, ((char *)t->stack+t->stacksize)))
             return NULL;
     
     /* finally insert it into the "new queue" where
@@ -410,32 +414,28 @@ void pth_yield(pth_t to)
     return;
 }
 
-/* switch a filedescriptor to non-blocking I/O mode */
-int pth_nonblocking(int fd)
+/* switch a filedescriptor's I/O mode */
+int pth_fdmode(int fd, int newmode)
 {
-    int mode;
-    int rv;
+    int fdmode;
+    int oldmode;
 
-#ifdef  O_NONBLOCK
-#define O_NONBLOCKING O_NONBLOCK
-#else
-#ifdef  O_NDELAY
-#define O_NONBLOCKING O_NDELAY
-#else
-#ifdef  FNDELAY
-#define O_NONBLOCKING FNDELAY
-#else
-#error "No O_NONBLOCK, O_NDELAY or FNDELAY flag available!"
-#endif
-#endif
-#endif
-    rv = TRUE;
-    mode = fcntl(fd, F_GETFL, NULL);
-    if (mode != -1 && (mode & O_NONBLOCKING) == 0) {
-        fcntl(fd, F_SETFL, mode|O_NONBLOCKING);
-        rv = FALSE;
-    }
-    return rv;
+    /* retrieve old mode (usually cheap) */
+    if ((fdmode = fcntl(fd, F_GETFL, NULL)) == -1)
+        oldmode = PTH_FDMODE_ERROR;
+    else if (fdmode & O_NONBLOCKING)
+        oldmode = PTH_FDMODE_NONBLOCK;
+    else 
+        oldmode = PTH_FDMODE_BLOCK;
+
+    /* set new mode (usually expensive) */
+    if (oldmode == PTH_FDMODE_BLOCK && newmode == PTH_FDMODE_NONBLOCK)
+        fcntl(fd, F_SETFL, (fdmode | O_NONBLOCKING));
+    if (oldmode == PTH_FDMODE_NONBLOCK && newmode == PTH_FDMODE_BLOCK)
+        fcntl(fd, F_SETFL, (fdmode & ~(O_NONBLOCKING)));
+
+    /* return old mode */
+    return oldmode;
 }
 
 /* wait for specific amount of time */
