@@ -21,7 +21,10 @@
 **  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 **  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
 */
-
+                             /* ``It is hard to fly with
+                                  the eagles when you work
+                                  with the turkeys.''
+                                          -- Unknown  */
 #include "pth_p.h"
 
 /*
@@ -39,7 +42,7 @@ int pth_mutex_init(pth_mutex_t *mutex)
     return TRUE;
 }
 
-int pth_mutex_acquire(pth_mutex_t *mutex, int try, pth_event_t ev_extra)
+int pth_mutex_acquire(pth_mutex_t *mutex, int tryonly, pth_event_t ev_extra)
 {
     static pth_key_t ev_key = PTH_KEY_INIT;
     pth_event_t ev;
@@ -70,8 +73,8 @@ int pth_mutex_acquire(pth_mutex_t *mutex, int try, pth_event_t ev_extra)
         return TRUE;
     }
 
-    /* should we just try? */
-    if (try)
+    /* should we just tryonly? */
+    if (tryonly)
         return_errno(FALSE, EBUSY); 
 
     /* else wait for mutex to become unlocked.. */
@@ -154,7 +157,7 @@ int pth_rwlock_init(pth_rwlock_t *rwlock)
     return TRUE;
 }
 
-int pth_rwlock_acquire(pth_rwlock_t *rwlock, int op, int try, pth_event_t ev_extra)
+int pth_rwlock_acquire(pth_rwlock_t *rwlock, int op, int tryonly, pth_event_t ev_extra)
 {
     /* consistency checks */
     if (rwlock == NULL)
@@ -165,17 +168,17 @@ int pth_rwlock_acquire(pth_rwlock_t *rwlock, int op, int try, pth_event_t ev_ext
     /* acquire lock */
     if (op == PTH_RWLOCK_RW) {
         /* read-write lock is simple */
-        if (!pth_mutex_acquire(&(rwlock->rw_mutex_rw), try, ev_extra))
+        if (!pth_mutex_acquire(&(rwlock->rw_mutex_rw), tryonly, ev_extra))
             return FALSE;
         rwlock->rw_mode = PTH_RWLOCK_RW;
     }
     else {
         /* read-only lock is more complicated to get right */
-        if (!pth_mutex_acquire(&(rwlock->rw_mutex_rd), try, ev_extra))
+        if (!pth_mutex_acquire(&(rwlock->rw_mutex_rd), tryonly, ev_extra))
             return FALSE;
         rwlock->rw_readers++;
         if (rwlock->rw_readers == 1) {
-            if (!pth_mutex_acquire(&(rwlock->rw_mutex_rw), try, ev_extra)) {
+            if (!pth_mutex_acquire(&(rwlock->rw_mutex_rw), tryonly, ev_extra)) {
                 rwlock->rw_readers--;
                 errno_preserve;
                 pth_mutex_release(&(rwlock->rw_mutex_rd));
@@ -294,3 +297,59 @@ int pth_cond_notify(pth_cond_t *cond, int broadcast)
     return TRUE;
 }
 
+/*
+**  Barriers
+*/
+
+int pth_barrier_init(pth_barrier_t *barrier, int threshold)
+{
+    if (barrier == NULL || threshold <= 0)
+        return_errno(FALSE, EINVAL);
+    if (!pth_mutex_init(&(barrier->br_mutex)))
+        return FALSE;
+    if (!pth_cond_init(&(barrier->br_cond)))
+        return FALSE;
+    barrier->br_state     = PTH_BARRIER_INITIALIZED;
+    barrier->br_threshold = threshold;
+    barrier->br_count     = threshold;
+    barrier->br_cycle     = FALSE;
+    return TRUE;
+}
+
+int pth_barrier_reach(pth_barrier_t *barrier)
+{
+    int cancel, cycle;
+    int rv;
+
+    if (barrier == NULL)
+        return_errno(FALSE, EINVAL);
+    if (!(barrier->br_state & PTH_BARRIER_INITIALIZED))
+        return_errno(FALSE, EINVAL);
+
+    if (!pth_mutex_acquire(&(barrier->br_mutex), FALSE, NULL))
+        return FALSE;
+    cycle = barrier->br_cycle;
+    if (--(barrier->br_count) == 0) {
+        /* last thread reached the barrier */
+        barrier->br_cycle   = !(barrier->br_cycle);
+        barrier->br_count   = barrier->br_threshold;
+        if ((rv = pth_cond_notify(&(barrier->br_cond), TRUE)))
+            rv = PTH_BARRIER_TAILLIGHT;
+    }
+    else {
+        /* wait until remaining threads have reached the barrier, too */
+        pth_cancel_state(PTH_CANCEL_DISABLE, &cancel);
+        if (barrier->br_threshold == barrier->br_count)
+            rv = PTH_BARRIER_HEADLIGHT;
+        else
+            rv = TRUE;
+        while (cycle == barrier->br_cycle) {
+            if (!(rv = pth_cond_await(&(barrier->br_cond), &(barrier->br_mutex), NULL)))
+                break;
+        }
+        pth_cancel_state(cancel, NULL);
+    }
+    pth_mutex_release(&(barrier->br_mutex));
+    return rv;
+}
+
