@@ -21,7 +21,8 @@
 **  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 **  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
 */
-
+                             /* Patient: Doctor, it hurts when I do this!
+                                Doctor: Well, then don't do it. */
 #include "pth_p.h"
 
 #if cpp
@@ -57,6 +58,7 @@ struct pth_st {
     char          *stack;                /* pointer to thread stack */
     unsigned int   stacksize;            /* size of thread stack */
     long          *stackguard;           /* stack overflow guard */
+    int            stackloan;            /* stack type */
     void        *(*start_func)(void *);  /* start routine */
     void          *start_arg;            /* start argument */
 
@@ -95,28 +97,30 @@ intern pth_t pth_tcb_alloc(unsigned int stacksize, void *stackaddr)
 {
     pth_t t;
 
-    if (stacksize < SIGSTKSZ)
+    if (stacksize > 0 && stacksize < SIGSTKSZ)
         stacksize = SIGSTKSZ;
-    if ((t = malloc(sizeof(struct pth_st))) == NULL)
+    if ((t = (pth_t)malloc(sizeof(struct pth_st))) == NULL)
         return NULL;
     t->stacksize  = stacksize;
     t->stack      = NULL;
     t->stackguard = NULL;
-    if (stacksize > 0) {
+    t->stackloan  = (stackaddr != NULL ? TRUE : FALSE);
+    if (stacksize > 0) { /* stacksize == 0 means "main" thread */
         if (stackaddr != NULL)
             t->stack = (char *)(stackaddr);
         else {
-            if ((t->stack = malloc(stacksize)) == NULL) {
+            if ((t->stack = (char *)malloc(stacksize)) == NULL) {
                 free(t);
                 return NULL;
             }
         }
-        if (PTH_STACKSGROWDOWN)
-            /* guard is at lowest address (alignment is guarrantied) */
-            t->stackguard = (long *)(t->stack); 
-        else
-            /* guard is at highest address (be careful with alignment) */
-            t->stackguard = (long *)(t->stack+(((stacksize/sizeof(long))-1)*sizeof(long)));
+#if PTH_STACKGROWTH < 0
+        /* guard is at lowest address (alignment is guarrantied) */
+        t->stackguard = (long *)((long)t->stack); /* double cast to avoid alignment warning */
+#else
+        /* guard is at highest address (be careful with alignment) */
+        t->stackguard = (long *)(t->stack+(((stacksize/sizeof(long))-1)*sizeof(long)));
+#endif
         *t->stackguard = 0xDEAD;
     }
     return t;
@@ -127,7 +131,7 @@ intern void pth_tcb_free(pth_t t)
 {
     if (t == NULL)
         return;
-    if (t->stack != NULL)
+    if (t->stack != NULL && !t->stackloan)
         free(t->stack);
     if (t->data_value != NULL)
         free(t->data_value);
