@@ -21,26 +21,21 @@
 **  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 **  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
 */
-
-/* protect us from vendor pthread stuff */
-#define _PTHREAD_H_
-#define _PTHREAD_H
-#define PTHREAD_H
-
-/* include Pth and various vendor headers */
-#define PTH_SYSCALL_SOFT FALSE
-#include "pth_p.h"
-
-/* reverse protection hack */
-#undef _PTHREAD_H_
-#undef _PTHREAD_H
-#undef PTHREAD_H
-
-/* now include our own pthread header */
+                             /* ``The nice thing about standards is that 
+                                  there are so many to choose from.  And if
+                                  you really don't like all the standards you
+                                  just have to wait another year until the one
+                                  arises you are looking for'' 
+                                                 -- Tannenbaum, 'Introduction 
+                                                    to Computer Networks' */
+/* include our own Pthread header */
 #define _PTHREAD_PRIVATE
 #include "pthread.h"
 #undef _PTHREAD_PRIVATE
 
+/* include Pth and various other vendor headers */
+#include "pth_p.h"
+
 /*
 **  HEADER STUFF
 */
@@ -642,7 +637,7 @@ int pthread_mutex_init(pthread_mutex_t *mutex, const pthread_mutexattr_t *attr)
     pthread_initialize();
     if (mutex == NULL)
         return_errno(EINVAL, EINVAL);
-    if ((m = malloc(sizeof(pth_mutex_t))) == NULL)
+    if ((m = (pth_mutex_t *)malloc(sizeof(pth_mutex_t))) == NULL)
         return errno;
     if (!pth_mutex_init(m))
         return errno;
@@ -765,7 +760,7 @@ int pthread_rwlock_init(pthread_rwlock_t *rwlock, const pthread_rwlockattr_t *at
     pthread_initialize();
     if (rwlock == NULL)
         return_errno(EINVAL, EINVAL);
-    if ((rw = malloc(sizeof(pth_rwlock_t))) == NULL)
+    if ((rw = (pth_rwlock_t *)malloc(sizeof(pth_rwlock_t))) == NULL)
         return errno;
     if (!pth_rwlock_init(rw))
         return errno;
@@ -890,7 +885,7 @@ int pthread_cond_init(pthread_cond_t *cond, const pthread_condattr_t *attr)
     pthread_initialize();
     if (cond == NULL)
         return_errno(EINVAL, EINVAL);
-    if ((cn = malloc(sizeof(pth_cond_t))) == NULL)
+    if ((cn = (pth_cond_t *)malloc(sizeof(pth_cond_t))) == NULL)
         return errno;
     if (!pth_cond_init(cn))
         return errno;
@@ -1049,16 +1044,16 @@ ssize_t _pthread_write(int fd, const void *buf, size_t nbytes)
     return pth_write(fd, buf, nbytes);
 }
 
-ssize_t _pthread_readv(int fd, const struct iovec *iovec, int iocnt)
+ssize_t _pthread_readv(int fd, const struct iovec *piovec, int iocnt)
 {
     pthread_initialize();
-    return pth_readv(fd, iovec, iocnt);
+    return pth_readv(fd, piovec, iocnt);
 }
 
-ssize_t _pthread_writev(int fd, const struct iovec *iovec, int iocnt)
+ssize_t _pthread_writev(int fd, const struct iovec *piovec, int iocnt)
 {
     pthread_initialize();
-    return pth_writev(fd, iovec, iocnt);
+    return pth_writev(fd, piovec, iocnt);
 }
 
 ssize_t _pthread_pread(int fd, void *buf, size_t nbytes, off_t offset)
