@@ -70,7 +70,6 @@ int main(int argc, char *argv[])
     pth_yield(NULL);
     
     evt = NULL;
-    pth_nonblocking(STDIN_FILENO);
     for (;;) {
         if (evt == NULL)
             evt = pth_event(PTH_EVENT_TIME, pth_timeout(10,0));
@@ -83,6 +82,10 @@ int main(int argc, char *argv[])
             fprintf(stderr, "main: timeout - repeating\n");
             continue;
         }
+        if (!FD_ISSET(STDIN_FILENO, &rfds)) {
+            fprintf(stderr, "main: Hmmmm... strange situation: bit not set\n");
+            exit(1);
+        }
         fprintf(stderr, "main: select returned %d\n", n);
         while (pth_read(STDIN_FILENO, &c, 1) > 0)
             fprintf(stderr, "main: read stdin '%c'\n", c);
